######################################################################################
######################################################################################

# Parent E. and Rivot E. - An introduction to Hierarchcial Modeling of Ecological Data
# Chapman et al.
# 2012

# BRUGS program for state-space modelling of A. Salmon life cycle
# Chapter 11 - section 11.2

######################################################################################
######################################################################################

rm(list = ls())

library(BRugs)
library(coda)
library(boa)
library(MASS)

# Create repertory "results" and "results_forecast" in the working directory
dir.create(path = "results", recursive = TRUE, showWarnings = TRUE)
dir.create(path = "results_forecast", recursive = TRUE, showWarnings = TRUE)

# load a function needed to save mcmc results
source("function.save.mcmc.R")

# MCMC sampling

n_thin <- 10
n_burnin <- 10000
n_iter <- 10000
n_chains <- 3
n_draws <- n_iter*n_chains




# ----------------------------------------------------------------------------
# ----------------------------------------------------------------------------
# Run model with all data
# ----------------------------------------------------------------------------
# ----------------------------------------------------------------------------


monitor <- c( "alpha", "log.alpha", "beta", "sigma_2", "exp.beta",
		"mean_egg_juv_surv",	
		"s01", "s12", "theta_Sm1", "p_Sm1",
		"mean_p_Eggs1", "mean_p_Sm1", "mean_p_Sp1", "mean_p_Sp11", "mean_p_Sp12",
		"ss11", "ss21", "delta_ss1", "theta_m1", "ss2",
		"Sp11", "Sp12", "Sp21", "Sp22", "Sp", "Sm1", "Sm2", "Sm",
		"Eggs", "J",
		"pf1", "pf2",
		"pi_sm", "pi_sp1", "pi_sp2"
		)

# Run OpenBUGS model

model  <- "model_SalmonLifeCycle.txt"
data   <- "data_SalmonLifeCycle.txt"
inits1 <- "inits1_SalmonLifeCycle.txt"
inits2 <- "inits2_SalmonLifeCycle.txt"
inits3 <- "inits3_SalmonLifeCycle.txt"
modelCheck(model)
modelData(data)
modelCompile(numChains=n_chains)
modelInits(c(inits1,inits2,inits3))
modelGenInits()

modelUpdate(numUpdates = n_burnin, thin = n_thin, overRelax = FALSE)
samplesSet(monitor)
# dicSet()
modelUpdate(numUpdates = n_iter, thin = n_thin, overRelax = FALSE)


# Save results
# -------------

setwd("./results")
# DIC <- dicStats()
# name <- "DIC"
# write.table(x=DIC, file = name, append = F, quote = F)
stats <- samplesStats("*")
name <- "stats"
write.table(x=stats, file = name, append = F, quote = F)
for(i in 1:length(monitor)) {function.save.mcmc(var=monitor[i],write=TRUE)}
setwd("../")




# ----------------------------------------------------------------------------
# ----------------------------------------------------------------------------
# Run model with missing data ("forecasting")
# ----------------------------------------------------------------------------
# ----------------------------------------------------------------------------

monitor <- c("Sp11", "Sm1")

model <- "model_SalmonLifeCycle.txt"
data <- "data_SalmonLifeCycle_forecast.txt"
inits1 <- "inits1_SalmonLifeCycle.txt"
inits2 <- "inits2_SalmonLifeCycle.txt"
inits3 <- "inits3_SalmonLifeCycle.txt"
modelCheck(model)
modelData(data)
modelCompile(numChains=n_chains)
modelInits(c(inits1,inits2,inits3))
modelGenInits()
modelUpdate(numUpdates = n_burnin, thin = n_thin, overRelax = FALSE)

samplesSet(monitor)
# dicSet()
modelUpdate(numUpdates = n_iter, thin = n_thin, overRelax = FALSE)

# Save results
# -------------

setwd("./results_forecast")
stats <- samplesStats("*")
name <- "stats"
write.table(x=stats, file = name, append = F, quote = F)
for(i in 1:length(monitor)) {function.save.mcmc(var=monitor[i],write=TRUE)}
setwd("../")






# --------------------------------------------------------------------------
# --------------------------------------------------------------------------
# Draw figures
# --------------------------------------------------------------------------
# --------------------------------------------------------------------------


data <- source("data_SalmonLifeCycle.txt")$value
n <- data$n

setwd("./results")

Sp <- as.matrix(read.table(file = "Sp", header = T, sep = ""))
Sp11 <- as.matrix(read.table(file = "Sp11", header = T, sep = ""))
Sp12 <- as.matrix(read.table(file = "Sp12", header = T, sep = ""))
Sm <- as.matrix(read.table(file = "Sm", header = T, sep = ""))
Sm1 <- as.matrix(read.table(file = "Sm1", header = T, sep = ""))
Sm2 <- as.matrix(read.table(file = "Sm2", header = T, sep = ""))
theta_Sm1 <- as.matrix(read.table(file = "theta_Sm1", header = T, sep = ""))
ss11 <- as.matrix(read.table(file = "ss11", header = T, sep = ""))
pf1 <- as.matrix(read.table(file = "pf1", header = T, sep = ""))
pf2 <- as.matrix(read.table(file = "pf2", header = T, sep = ""))
pi_sm <- as.matrix(read.table(file = "pi_sm", header = T, sep = ""))
pi_sp1 <- as.matrix(read.table(file = "pi_sp1", header = T, sep = ""))
Eggs <- as.matrix(read.table(file = "Eggs", header = T, sep = ""))
J <- as.matrix(read.table(file = "J", header = T, sep = ""))

alpha <- as.vector(read.table(file = "alpha", header = T, sep = ""))
beta <- as.vector(read.table(file = "beta", header = T, sep = ""))
sigma_2 <- as.vector(read.table(file = "sigma_2", header = T, sep = ""))
theta_m1 <- as.vector(read.table(file = "theta_m1", header = T, sep = ""))
s01 <- as.vector(read.table(file = "s01", header = T, sep = ""))
s12 <- as.vector(read.table(file = "s12", header = T, sep = ""))
delta_ss1 <- as.vector(read.table(file = "delta_ss1", header = T, sep = ""))
ss2 <- as.vector(read.table(file = "ss2", header = T, sep = ""))

setwd("../")




# --------------------------------------------------------------------------
# Boxplot of time series
# --------------------------------------------------------------------------

n.proj.max = 3
size.labels = 1
box.size = 0.5
size.text = 1.3

col.obs <- "grey85"
col.proj <- "grey55"
col = c(rep(col.obs, times=n),rep(col.proj, times=n.proj.max))
col2 = c(rep(col.proj, times=n),rep(col.proj, times=n.proj.max))




# Smolts and spawners run sizes
# ----------------------------------------------
# ----------------------------------------------

def.par <- par(no.readonly = TRUE)

# windows()

res <- 6
name_figure <- "EstimatesSpawnersSmolts.png"
png(filename = name_figure, height = 1000*res, width = 650*res, res=72*res)

par(mfrow = c(2,1), mar=c(5,5,3,1), bty="n")

# Smolts Sm
# ----------------------------------------------

X <- Sm
n.proj <- 0
years <- 1984:(1984+(n+n.proj)-1)

x.min = 1 ; x.max = (n+n.proj)+1
y.min = 0 ; y.max = 2500
line.y = y.max*2
y.label = "Smolts"
x.label = "Years"
title = "a)"

boxplot(	X[,1], 
		xaxt = "n", yaxt = "n", 
		xlim = c(x.min,x.max), ylim = c(y.min,y.max), 
		at = 1, outpch = NA, boxwex = box.size, col = col[1]) 
for (i in 2:(n+n.proj)) {
boxplot(X[,i], xaxt = "n", yaxt = "n", at = i, add = T, outpch = NA, boxwex = box.size, col = col[i]) }

# points(x=x.min:(x.max-1),y=rep(line.y,n),col="red",type="l",lty=2,lwd="1")
axis(side =1, at=1:(n+n.proj), labels = years, las=3, cex.axis=size.labels)
axis(side =2, cex.axis=size.labels)
# mtext(x.label, line = 3, side = 1, cex = size.text)
mtext(y.label, line = 3, side = 2, cex = size.text)
mtext(title, line = 1.5, side = 3, adj=0, cex = size.text+0.1)


# Spawners Sp
# ----------------------------------------------

X <- Sp
n.proj <- 0
years <- 1984:(1984+(n+n.proj)-1)

x.min = 1 ; x.max = (n+n.proj)+1
y.min = 0 ; y.max = 600
line.y = y.max*2
y.label = "Spawners"
x.label = "Years"
title = "b)"

boxplot(	X[,1], 
		xaxt = "n", yaxt = "n", 
		xlim = c(x.min,x.max), ylim = c(y.min,y.max), 
		at = 1, outpch = NA, boxwex = box.size, col = col[1]) 
for (i in 2:(n+n.proj)) {
boxplot(X[,i], xaxt = "n", yaxt = "n", at = i, add = T, outpch = NA, boxwex = box.size, col = col[i]) }

# points(x=x.min:(x.max-1),y=rep(line.y,n),col="red",type="l",lty=2,lwd="1")
axis(side =1, at=1:(n+n.proj), labels = years, las=3, cex.axis=size.labels)
axis(side =2, cex.axis=size.labels)
mtext(x.label, line = 3, side = 1, cex = size.text)
mtext(y.label, line = 3, side = 2, cex = size.text)
mtext(title, line = 1.5, side = 3, adj=0, cex = size.text+0.1)

par(def.par)

dev.off()






# Trapping efficiencies
# ----------------------------------------------
# ----------------------------------------------

def.par <- par(no.readonly = TRUE)

# windows()

res <- 6
name_figure <- "EstimatesTrapEff.png"
png(filename = name_figure, height = 1000*res, width = 650*res, res=72*res)

par(mfrow = c(2,1), mar=c(5,5,3,1), bty="n")

# Trapping eff. Smolts
# ----------------------------------------------

X <- pi_sm
n.proj <- 1
# years <- 1984:(1984+(n+n.proj)-1)
years <- c(1984:(1984+n-1),"Pred.")

x.min = 1 ; x.max = (n+n.proj)+1
y.min = 0 ; y.max = 1
line.y = y.max*2
y.label = "Trapp. eff. - Smolts"
x.label = "Years"
title = "a)"

boxplot(	X[,1], 
		xaxt = "n", yaxt = "n", 
		xlim = c(x.min,x.max), ylim = c(y.min,y.max), 
		at = 1, outpch = NA, boxwex = box.size, col = col[1]) 
for (i in 2:(n+n.proj)) {
boxplot(X[,i], xaxt = "n", yaxt = "n", at = i, add = T, outpch = NA, boxwex = box.size, col = col[i]) }

# points(x=x.min:(x.max-1),y=rep(line.y,n),col="red",type="l",lty=2,lwd="1")
axis(side =1, at=1:(n+n.proj), labels = years, las=3, cex.axis=size.labels)
axis(side =2, cex.axis=size.labels)
# mtext(x.label, line = 3, side = 1, cex = size.text)
mtext(y.label, line = 3, side = 2, cex = size.text)
mtext(title, line = 1.5, side = 3, adj=0, cex = size.text+0.1) 

# Trapping eff. Spawners
# ----------------------------------------------

X <- pi_sp1
n.proj <- 1
# years <- 1984:(1984+(n+n.proj)-1)
years <- c(1984:(1984+n-1),"Pred.")

x.min = 1 ; x.max = (n+n.proj)+1
y.min = 0 ; y.max = 1
line.y = y.max*2
y.label = "Trapp. eff. - Spawners"
x.label = "Years"
title = "b)"

boxplot(	X[,1], 
		xaxt = "n", yaxt = "n", 
		xlim = c(x.min,x.max), ylim = c(y.min,y.max), 
		at = 1, outpch = NA, boxwex = box.size, col = col[1]) 
for (i in 2:(n+n.proj)) {
boxplot(X[,i], xaxt = "n", yaxt = "n", at = i, add = T, outpch = NA, boxwex = box.size, col = col[i]) }

# points(x=x.min:(x.max-1),y=rep(line.y,n),col="red",type="l",lty=2,lwd="1")
axis(side =1, at=1:(n+n.proj), labels = years, las=3, cex.axis=size.labels)
axis(side =2, cex.axis=size.labels)
mtext(x.label, line = 3, side = 1, cex = size.text)
mtext(y.label, line = 3, side = 2, cex = size.text)
mtext(title, line = 1.5, side = 3, adj=0, cex = size.text+0.1) 

par(def.par)

dev.off()





# Eggs --> Juveniles survival rates with fitted Ricker SR
# -------------------------------------------------------

def.par <- par(no.readonly = TRUE)

# windows()

res <- 6
name_figure <- "RickerRelationship.png"
png(filename = name_figure, height = 500*res, width = 650*res, res=72*res)

par(mfrow = c(1,1), mar = c(5,5,1,1), bty="n")

x <- apply(Eggs,MARGIN=2,FUN=median)
y <- apply(J,MARGIN=2,FUN=median)

a <- median(alpha$alpha)*25229
b <- median(beta$beta)

x.line <- seq(from = 0, to = max(x)*1.2, length.out = 100)
y.line <- a*x.line*exp(-b*x.line)

plot(x, y, ylab = "", xlab = "", xaxt="n", yaxt="n", main="", col = "black", pch = 19, cex =1, xlim = c(0,max(x)*1.2), ylim = c(0,max(y)*1.1))
points(x.line, y.line, ylab = "", xlab = "", xaxt="n", yaxt="n", main="", col = "grey40", type = "l", lty = 1, lwd = 2)
text(x,y,labels=1984:2001, pos = c(4,4,4,4,4,3,4,2,4,2,2,2,4,4,4,4,4,2), cex = 0.8)

axis(side = 1, tick = T, cex.axis=1, las=1)
axis(side = 2, tick = T, cex.axis=1)
mtext(side=1, "Eggs", bty="n", line=3, cex = size.text)
mtext(side=2, "0+ Juveniles", bty="n", line=3, cex = size.text)

par(def.par)

dev.off()






# Smoltifiction probability and survival rates
# ----------------------------------------------
# ----------------------------------------------

def.par <- par(no.readonly = TRUE)

# windows()

res <- 6
name_figure <- "ProbaSm1_PostSmoltsSurvivalRates.png"
png(filename = name_figure, height = 1000*res, width = 650*res, res=72*res)

par(mfrow = c(2,1), mar=c(5,5,3,1), bty="n")

# Smoltification probability
# ----------------------------------------------

X <- theta_Sm1
n.proj <- 1
# years <- 1984:(1984+(n+n.proj)-1)
years <- c(1984:(1984+n-1),"Pred.")

x.min = 1 ; x.max = (n+n.proj)+1
y.min = 0.4 ; y.max = 1
line.y = y.max*2
y.label = "Prob. to smoltify as 1+ Smolt"
x.label = "Years"
title = "a)"

boxplot(	X[,1], 
		xaxt = "n", yaxt = "n", 
		xlim = c(x.min,x.max), ylim = c(y.min,y.max), 
		at = 1, outpch = NA, boxwex = box.size, col = col[1]) 
for (i in 2:(n+n.proj)) {
boxplot(X[,i], xaxt = "n", yaxt = "n", at = i, add = T, outpch = NA, boxwex = box.size, col = col[i]) }

# points(x=x.min:(x.max-1),y=rep(line.y,n),col="red",type="l",lty=2,lwd="1")
axis(side =1, at=1:(n+n.proj), labels = years, las=3, cex.axis=size.labels)
axis(side =2, cex.axis=size.labels)
mtext(x.label, line = 3, side = 1, cex = size.text)
mtext(y.label, line = 3, side = 2, cex = size.text)
mtext(title, line = 1.5, side = 3, adj=0, cex = size.text+0.1)


# Sea Survival rate ss11
# ----------------------------------------------

X <- ss11
n.proj <- 1
# years <- 1984:(1984+(n+n.proj)-1)
years <- c(1984:(1984+n-1),"Pred.")

x.min = 1 ; x.max = (n+n.proj)+1
y.min = 0 ; y.max = 1
line.y = y.max*2
y.label = "Sea survival rates"
x.label = "Years"
title = "b)"

boxplot(	X[,1], 
		xaxt = "n", yaxt = "n", 
		xlim = c(x.min,x.max), ylim = c(y.min,y.max), 
		at = 1, outpch = NA, boxwex = box.size, col = col[1]) 
for (i in 2:(n+n.proj)) {
boxplot(X[,i], xaxt = "n", yaxt = "n", at = i, add = T, outpch = NA, boxwex = box.size, col = col[i]) }

# points(x=x.min:(x.max-1),y=rep(line.y,n),col="red",type="l",lty=2,lwd="1")
axis(side =1, at=1:(n+n.proj), labels = years, las=3, cex.axis=size.labels)
axis(side =2, cex.axis=size.labels)
mtext(x.label, line = 3, side = 1, cex = size.text)
mtext(y.label, line = 3, side = 2, cex = size.text)
mtext(title, line = 1.5, side = 3, adj=0, cex = size.text+0.1)

par(def.par)

dev.off()




# --------------------------------------------------------------------------
# Posterior estimates of parameters with correlations
# --------------------------------------------------------------------------

def.par <- par(no.readonly = TRUE)

# windows()

res <- 6
name_figure <- "PostParamSalmon.png"
png(filename = name_figure, height = 500*res, width = 500*res, res=72*res)

nb.draws <- 10000

x3 <- s01[1:nb.draws,1]
x4 <- s12[1:nb.draws,1]
x5 <- delta_ss1[1:nb.draws,1]
x6 <- theta_m1[1:nb.draws,1]
x7 <- ss2[1:nb.draws,1]
param <- cbind(x3,x4,x5,x6,x7)
labels <- c(expression(gamma[paste(0,"+")]), expression(gamma[Parr1]), 
            expression(delta[gamma]), expression(theta[m1]), expression(gamma[Res]))

panel.dens <- function(x, ...)
{
    usr <- par("usr"); on.exit(par(usr))
    par(usr = c(usr[1:2], 0, 1.5) )
    h <- density(x,adjust=1)
    h$y<-h$y/max(h$y)
    xlim=range(h$x)
    lines(h, col="black", lwd=2, ...)
}

## put (absolute) correlations on the upper panels
panel.cor <- function(x, y, digits=2, prefix="", cex.cor)
{
    usr <- par("usr"); on.exit(par(usr))
    par(usr = c(0, 1, 0, 1))
    r <- (cor(x, y))
    txt <- format(c(r, 0.123456789), digits=digits)[1]
    txt <- paste(prefix, txt, sep="")
    if(missing(cex.cor)) cex <- 0.8/strwidth(txt)
    text(0.5, 0.5, txt, cex = 1.5)
}

par(pch='.', cex=0.5)

pairs( param,labels=labels, 
	 lower.panel=panel.smooth,
	 diag.panel=panel.dens, 
	 upper.panel=panel.cor,
	 cex.labels = 1.5, font.labels=1.5)

par(def.par)

dev.off()




# Forecast for 1+ smolts and spawners
# ----------------------------------------------
# ----------------------------------------------

setwd("./results_forecast")
Sp11_forecast <- as.matrix(read.table(file = "Sp11", header = T, sep = ""))
Sm1_forecast <- as.matrix(read.table(file = "Sm1", header = T, sep = ""))
setwd("../")

data <- source("data_SalmonLifeCycle.txt")$value
n <- data$n




def.par <- par(no.readonly = TRUE)

# windows()

res <- 6
name_figure <- "Forecast.png"
png(filename = name_figure, height = 1000*res, width = 650*res, res=72*res)

par(mfrow = c(2,1), mar=c(5,5,3,1), bty="n")

# Smolts Sm1
# ----------------------------------------------

X <- Sm1
X2 <- Sm1_forecast

years <- 1984:(1984+n-1)

x.min = 1 ; x.max = n+1
y.min = 0 ; y.max = 2000
y.label = "1+ Smolts"
x.label = "Years"
title = "a)"

boxplot(	X[,1], 
		xaxt = "n", yaxt = "n", 
		xlim = c(x.min,x.max), ylim = c(y.min,y.max), 
		at = 1, outpch = NA, boxwex=box.size, col = col[1]) 
for (i in 2:n) {
boxplot(X[,i], xaxt = "n", yaxt = "n", at = i, add = T, outpch = NA, boxwex=box.size, col = col[i]) }

for (i in n:n) {
boxplot(X2[,i], xaxt = "n", yaxt = "n", at = i+0.4, add = T, outpch = NA, boxwex=box.size, col = col2[i]) }

axis(side =1, at=c(1:(n-1),n+0.2), labels = years, las=3, cex.axis=size.labels)
axis(side =2, cex.axis=size.labels)
mtext(x.label, line = 3, side = 1, cex = size.text)
mtext(y.label, line = 3, side = 2, cex = size.text)
mtext(title, line = 1.5, side = 3, adj=0, cex = size.text + 0.1)


# Spawners Sp11
# ----------------------------------------------

X <- Sp11
X2 <- Sp11_forecast

years <- 1984:(1984+n-1)

x.min = 1 ; x.max = n+1
y.min = 0 ; y.max = 500
y.label = "Spawners Sp11"
x.label = "Years"
title = "b)"

boxplot(	X[,1], 
		xaxt = "n", yaxt = "n", 
		xlim = c(x.min,x.max), ylim = c(y.min,y.max), 
		at = 1, outpch = NA, boxwex=box.size, col = col[1]) 
for (i in 2:n) {
boxplot(X[,i], xaxt = "n", yaxt = "n", at = i, add = T, outpch = NA, boxwex=box.size, col = col[i]) }

for (i in n:n) {
boxplot(X2[,i], xaxt = "n", yaxt = "n", at = i+0.4, add = T, outpch = NA, boxwex=box.size, col = col2[i]) }

axis(side =1, at=c(1:(n-1),n+0.2), labels = years, las=3, cex.axis=size.labels)
axis(side =2, cex.axis=size.labels)
mtext(x.label, line = 3, side = 1, cex = size.text)
mtext(y.label, line = 3, side = 2, cex = size.text)
mtext(title, line = 1.5, side = 3, adj=0, cex = size.text + 0.1)

par(def.par)

dev.off()

